package captiveportal

import (
	"github.com/pkg/errors"
	"gitlab.com/wpetit/goweb/service"
)

const (
	ServiceName service.Name = "captiveportal"
)

func ServiceProvider(redirectURL string, identifier Identifier, opts ...OptionsFunc) service.Provider {
	srv := New(redirectURL, identifier, opts...)
	return func(ctn *service.Container) (interface{}, error) {
		return srv, nil
	}
}

// From retrieves the service in the given container
func From(container *service.Container) (*Service, error) {
	service, err := container.Service(ServiceName)
	if err != nil {
		return nil, errors.Wrapf(err, "error while retrieving '%s' service", ServiceName)
	}

	srv, ok := service.(*Service)
	if !ok {
		return nil, errors.Errorf("retrieved service is not a valid '%s' service", ServiceName)
	}

	return srv, nil
}

// Must retrieves the service in the given container or panic otherwise
func Must(container *service.Container) *Service {
	srv, err := From(container)
	if err != nil {
		panic(err)
	}

	return srv
}
